'use strict';

const chalk = require('chalk');
const dateparser = require('dateparser');
const PullRequest = require('./PullRequest');

class Auditor {

  /**
   * Creates an instance of Auditor.
   * @param {Object} bitbucketApiClient BitbucketApiClient object
   * @param {Object} definitionOfStale
   * @memberof Auditor
   */
  constructor(bitbucketApiClient, definitionOfStale) {
    this.bitbucketApiClient = bitbucketApiClient;
    this.stalePrs = [];
    this.definitionOfStale = definitionOfStale;
  }

  auditProject(project) {
    let projectPromises = [];

    projectPromises.push(this._processProjectRepos(project));

    return Promise.all(projectPromises);
  }

  _processProjectRepos(project) {
    // Fetch Repo Info Below
    return new Promise((resolve, reject) => {
      this.bitbucketApiClient.getRepos(project.key).then((repos) => {
        this._auditProjectRepos(project, repos).then(function() {
          console.log(chalk.green(chalk.bold(project.key) + ' project audit complete'));
          resolve();
        });
      }).catch(function(error) {
        reject(error);
      });
    });
  }

  _auditProjectRepos(project, repos) {
    let repoPromises = [];

    repos.forEach((repo) => {
      if (project.excludedRepos.indexOf(repo.slug) === -1) {
        repoPromises.push(this._auditRepo(project, repo));
      }
    });

    return Promise.all(repoPromises);
  }

  _auditRepo(project, repo) {
    let repoPromises = [];

    repoPromises.push(this._getPullRequests(project, repo));

    return Promise.all(repoPromises);
  }

  _getPullRequests(project, repo) {
    return new Promise((resolve, reject) => {
      this.bitbucketApiClient._getPullRequests(repo.project.key, repo.slug).then((pullRequests) => {
        this.stalePrs = this.stalePrs.concat(this._auditPullRequest(project, repo, pullRequests));
        resolve();
      }).catch(function(error) {
        console.log(error);
        reject(error);
      });
    });
  }

  _auditPullRequest(project, repo, pullRequests) {
    let repoName = repo.hasOwnProperty('slug') ? repo.slug : '';
    let definitionOfStale = project.hasOwnProperty('definitionOfStale') ? project.definitionOfStale : this.definitionOfStale;
    let allowedAgeInMillisecs = dateparser.parse(definitionOfStale).value;
    let stalePrs = [];

    for (let pullRequest of pullRequests) {
      let age = (this._getTodaysDate() - pullRequest.updatedDate);

      if (age > allowedAgeInMillisecs) {
        let stalePr = new PullRequest(project.key, repoName, pullRequest);
        stalePrs.push(stalePr);
      }
    }

    return stalePrs;
  }

  _getTodaysDate() {
    return new Date().getTime();
  }

  getStalePrs() {
    return this.stalePrs;
  }
}

module.exports = Auditor;
